#!/usr/bin/env python3

import sys
import os
import subprocess

trimmomatic_path = '/home/rui/anaconda3/pkgs/trimmomatic-0.36-5/share/trimmomatic-0.36-5/trimmomatic.jar'
threads = '16'
truseq3_pe = 'ILLUMINACLIP:/home/rui/anaconda3/pkgs/trimmomatic-0.36-5/share/trimmomatic-0.36-5/adapters/TruSeq3-PE.fa:2:30:10'
truseq3_se = 'ILLUMINACLIP:/home/rui/anaconda3/pkgs/trimmomatic-0.36-5/share/trimmomatic-0.36-5/adapters/TruSeq3-SE.fa:2:30:10'
minlen = 'MINLEN:36'

if len(sys.argv) != 3:
    print("Error with arguments. Usage python3 trim.py <INPUT_DIR> <OUTPUT_DIR>")
    quit()
print('Trimming fastq files...\n')

input_dir = sys.argv[1]
output_dir = sys.argv[2]
os.makedirs(output_dir, exist_ok=True)

fastq_files = []

for file in os.listdir(input_dir):
    if file.endswith('.fq.gz') or file.endswith('.fastq.gz'):
        fastq_files.append(file)
fastq_files.sort()
while len(fastq_files) > 0:
    file = fastq_files.pop(0)

    if '_1.' in file:
        pair_file = file.replace('_1.', '_2.')
        if pair_file in fastq_files:
            fastq_files.remove(pair_file)

            ina = os.path.join(input_dir, file)
            inb = os.path.join(input_dir, pair_file)

            outap = file.replace('.fq.gz', '.trimmed.paired.fq.gz') if file.endswith('.fq.gz') else file.replace('.fastq.gz', '.trimmed.paired.fastq.gz')
            outau = file.replace('.fq.gz', '.trimmed.unpaired.fq.gz') if file.endswith('.fq.gz') else file.replace('.fastq.gz', '.trimmed.unpaired.fastq.gz')
            outbp = pair_file.replace('.fq.gz', '.trimmed.paired.fq.gz') if pair_file.endswith('.fq.gz') else pair_file.replace('.fastq.gz', '.trimmed.paired.fastq.gz')
            outbu = pair_file.replace('.fq.gz', '.trimmed.unpaired.fq.gz') if pair_file.endswith('.fq.gz') else pair_file.replace('.fastq.gz', '.trimmed.unpaired.fastq.gz')

            outap = os.path.join(output_dir, outap)
            outau = os.path.join(output_dir, outau)
            outbp = os.path.join(output_dir, outbp)
            outbu = os.path.join(output_dir, outbu)

            if not os.path.exists(outap):
                print('Submitting pair %s %s' % (file, pair_file))
                command = ['java', '-jar', trimmomatic_path, 'PE', '-threads', threads, ina, inb, outap, outau, outbp, outbu, truseq3_pe, 'LEADING:3', 'TRAILING:3', 'SLIDINGWINDOW:4:15', minlen]
                subprocess.call(command)
            else:
                print('Found', outap, 'skipping')
        else:
            outfile = file.replace('.fq.gz', '.trimmed.fq.gz') if file.endswith('.fq.gz') else file.replace('.fastq.gz', '.trimmed.fastq.gz')
            if not os.path.exists(os.path.join(output_dir, outfile)):
                print('Submitting single $s' % file)
                command = ['java', '-jar', trimmomatic_path, 'SE', '-threads', threads,  os.path.join(input_dir, file), os.path.join(output_dir, outfile), truseq3_se, 'LEADING:3', 'TRAILING:3', 'SLIDINGWINDOW:4:15', minlen]
                subprocess.call(command)
            else:
                print('Found', outfile, 'skipping')
    else:
        outfile = file.replace('.fq.gz', '.trimmed.fq.gz') if file.endswith('.fq.gz') else file.replace('.fastq.gz', '.trimmed.fastq.gz')
        if not os.path.exists(os.path.join(output_dir, outfile)):
            print('Submitting single %s' % file)
            command = ['java', '-jar', trimmomatic_path, 'SE', '-threads', threads,  os.path.join(input_dir, file), os.path.join(output_dir, outfile), truseq3_se, 'LEADING:3', 'TRAILING:3', 'SLIDINGWINDOW:4:15', minlen]
            subprocess.call(command)
        else:
            print('Found', outfile, 'skipping')
