#!/usr/bin/env python3

import sys
import os
import subprocess

threads = '16'

if len(sys.argv) != 4:
    print("Error with arguments. Usage python3 alignment.py <INDEXING_DIR> <TRIMMED_FASTQ_DIR> <OUTPUT_DIR>")
    quit()
print('Mapping reads with HISAT2...')

indexing_input = sys.argv[1]
trimmed_fastq_input = sys.argv[2]
alignment_output = sys.argv[3]
os.makedirs(alignment_output, exist_ok=True)

pe_files = []
se_files = []

for file in os.listdir(trimmed_fastq_input):
    if file.endswith('.paired.fq.gz') or file.endswith('.paired.fastq.gz'):
        pe_files.append(file)
    elif not (file.endswith('.unpaired.fq.gz') or file.endswith('.unpaired.fastq.gz')):
        se_files.append(file)

pe_files.sort()
se_files.sort()

for pe_file in pe_files:
    if '_1.trimmed.paired.' in pe_file:
        pair_file = pe_file.replace('_1.trimmed.paired.', '_2.trimmed.paired.')
        output_sam = pe_file.replace('_1.trimmed.paired.fq.gz', '').replace('_1.trimmed.paired.fastq.gz', '') + '.sam'
        output_stats = pe_file.replace('_1.trimmed.paired.fq.gz', '').replace('_1.trimmed.paired.fastq.gz', '') + '.stats'
        output_sam = os.path.join(alignment_output, output_sam)
        output_stats = os.path.join(alignment_output, output_stats)
        forward = os.path.join(trimmed_fastq_input, pe_file)
        reverse = os.path.join(trimmed_fastq_input, pair_file)
        if not os.path.exists(output_sam):
            print('Submitting pair %s, %s' % (pe_file, pair_file))
            command = ['hisat2', '-p', threads, '-x', indexing_input, '-1', forward, '-2', reverse, '-S', output_sam, '2>', output_stats]
            subprocess.call(command)
        else:
            print('Output exists, skipping', pe_file)

for se_file in se_files:
    output_sam = se_file.replace('.trimmed.fq.gz', '').replace('.trimmed.fastq.gz', '') + '.sam'
    output_stats = se_file.replace('.trimmed.fq.gz', '').replace('.trimmed.fastq.gz', '') + '.stats'
    output_sam = os.path.join(alignment_output, output_sam)
    output_stats = os.path.join(alignment_output, output_stats)
    if not os.path.exists(output_sam):
        print('Submitting single %s' % se_file)
        command = ['hisat2', '-p', threads, '-x', indexing_input, '-U', os.path.join(trimmed_fastq_input, se_file), '-S', output_sam, '2>', output_stats]
        subprocess.call(command)
    else:
        print('Output exists, skipping', se_file)
