#!/usr/bin/env python3

import sys
import os
import re

alignment_cutoff = 65
htseq_cutoff = 40

if len(sys.argv) != 3:
    print("Error with arguments. Usage python3 check_quality.py <ALIGNMENT_DIR> <HTSEQ_DIR>")
    quit()

print('Checking quality of samples based on HISAT2 and HTSEQ-COUNT mapping statistics')
alignment_input = sys.argv[1]
htseq_input = sys.argv[2]

stats_files = [os.path.join(alignment_input, stats_file) for stats_file in os.listdir(alignment_input) if os.path.isfile(os.path.join(alignment_input, stats_file)) and stats_file.endswith('.stats')]
for stats_file in stats_files:
    re_mapped = re.compile('\t(.*)% overall alignment rate')
    with open(stats_file, 'r') as f:
        lines = '\t'.join(f.readlines())
        hits = re_mapped.search(lines)
        if hits:
            value = float(hits.group(1))
            if value >= alignment_cutoff:
                continue
            else:
                print('Warning: samples with insufficient quality (HISAT2) detected:', stats_file, file=sys.stderr)

htseq_files = [os.path.join(htseq_input, htseq_file) for htseq_file in os.listdir(htseq_input) if htseq_file.endswith('.htseq')]
quality_fields = ['__no_feature', '__ambiguous', '__too_low_aQual', '__not_aligned', '__alignment_not_unique']
for htseq_file in htseq_files:
    values = {}
    with open(htseq_file) as f:
        mapped_reads = 0
        for line in f:
            gene, value = line.strip().rsplit(maxsplit=1)
            if gene not in quality_fields:
                mapped_reads += int(value)
            else:
                values[gene] = int(value)
        total = sum([mapped_reads, values['__no_feature'] if '__no_feature' in values.keys() else 0, values['__ambiguous'] if '__ambiguous' in values.keys() else 0])
        percentage_mapped = ((mapped_reads * 100) / total) if total > 0 else 0
        if percentage_mapped >= htseq_cutoff:
            continue
        else:
            print('WARNING: sample with insufficient quality (HTSEQ-Count) detected:', htseq_file, file=sys.stderr)
