#!/usr/bin/env python3

import sys
import os

exp_matrix_output = './exp_matrix.txt'

if len(sys.argv) != 2:
    print("Error with arguments. Usage python3 matrix.py <HTSEQ_DIR>")
    quit()
htseq_input = sys.argv[1]

# htseq_to_matrix
htseq_files = [htseq_file for htseq_file in os.listdir(htseq_input) if htseq_file.endswith('.htseq')]
counts = {}
for file in htseq_files:
    full_path = os.path.join(htseq_input, file)
    with open(full_path, 'r') as f:
        for row in f:
            gene_id, count = row.strip().split('\t')
            if gene_id not in counts.keys():
                counts[gene_id] = {}
            counts[gene_id][file] = count

os.makedirs(os.path.dirname(exp_matrix_output), exist_ok=True)
with open(exp_matrix_output, "w") as f_out:
    header = '\t'.join(htseq_files)
    print('gene\t' + header, file = f_out)

    bad_fileds = ['no_feature', 'ambiguous', 'too_low_aQual', 'not_aligned', 'alignment_not_unique']
    for gene_id in counts:
        values = []
        for f in htseq_files:
            if f in counts[gene_id].keys():
                values.append(counts[gene_id][f])
        if all([x not in gene_id for x in bad_fileds]):
            print(gene_id.strip() + '\t' + '\t'.join(values), file=f_out)
print('Done\n\n')

