#!/usr/bin/env python3

import sys
import os

exp_matrix_rpkm_output = './exp_matrix.tpm.txt'


if len(sys.argv) != 3:
    print("Error with arguments. Usage python3 matrix.py <MATRIX_FILE> <CDS_FASTA_FILE>")
    quit()

exp_matrix_input = sys.argv[1]
fasta_file = sys.argv[2]


# read_matrix
data = {}
with open(exp_matrix_input, 'r') as f:
    header = f.readline().strip().split('\t')
    conditions = header[1:]
    for row in f:
        parts = row.strip().split('\t')
        gene_id = parts[0]
        expression_values = parts[1:]
        gene_data = {}
        for condition, read_counts in zip(conditions, expression_values):
            gene_data[condition] = read_counts
        data[gene_id] = gene_data

# normalize_matrix_length
print('Reading FASTA file:' + fasta_file + '...', file=sys.stderr)

name = ''
sequences = {}
sequence = []
count = 1
f = open(fasta_file, 'r')
for line in f:
    line = line.rstrip()
    if line.startswith('>'):
        if not name == '':
            sequences[name] = ''.join(sequence)
            count += 1
        name = line.lstrip('>').split()[0]
        sequence = []
    else:
        sequence.append(line)
sequences[name] = ''.join(sequence)
print('Done! (found ', count, ' sequence)', file=sys.stderr)

fasta_lengths = {}
length_normalized_data = {}
for gene_id, sequence in sequences.items():
    lenseq = len(sequence)/1000
    fasta_lengths[gene_id] = lenseq
for gene_id in data:
    if gene_id in fasta_lengths:
        if fasta_lengths[gene_id] > 0:
            length_normalized_data[gene_id] = {}
            for condition in data[gene_id]:
                length_normalized_data[gene_id][condition] = float(data[gene_id][condition]) / fasta_lengths[gene_id]
        else:
            print('no sequence', gene_id)
    else:
        print('no sequence', gene_id)

# normalize_matrix_counts
read_counts = {}
for condition in conditions:
    total = 0
    for gene_id in length_normalized_data:
        total += int(length_normalized_data[gene_id][condition])
    read_counts[condition] = total
normalized_data = {}
for gene_id in length_normalized_data:
    gene_normalized_data = {}
    for condition in conditions:
        if read_counts[condition] != 0:
            gene_normalized_data[condition] = (int(length_normalized_data[gene_id][condition]) * 1000000) / read_counts[condition]
        else:
            gene_normalized_data[condition] = 0
    normalized_data[gene_id] = gene_normalized_data

# write_matrix
os.makedirs(os.path.dirname(exp_matrix_rpkm_output), exist_ok=True)
with open(exp_matrix_rpkm_output, 'w') as f_norm:
    header = '\t'.join(conditions)
    print('gene\t' + header, file=f_norm)
    for gene_id in normalized_data:
        nv = []
        for condition in conditions:
            nv.append(str(normalized_data[gene_id][condition]))
        joined_values = '\t'.join(nv)
        print(gene_id + '\t' + joined_values, file=f_norm)
